%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Aircraft_Collocation_Procedure.m                    by:  Joseph Schlosser
%                                                revised:  08 Aug 2022 
%                                    language (revision):  MATLAB (2020a update 4)
% 
% Description: Collocate data using time, latitude, and longitude data from two different datasets. Each of the required 
% inputs for this code are described as follows:
% 1) Aircraft1_Filename: String or character containing the name of the .ict file with the data that will be collocated 
%    with the primary data.
% 2) Aircraft1_Filename: String or character containing the name of the .ict file with the data that will serve as the 
%    primary data.
% 3) time_sep_min: User defined maximum temporal separation (minutes) between primary data and data to be collocated. 
% 4) distance_sep_km: User defined maximum horizontal separation (km) between primary data and data to be collocated. 
% 5) N1: number of allowed independant time segments that can be collocated with each data point in the primary data. 
%
% the output of this code is a MATLAB structure containing the primary data's time series, and two matricies conatining
% number of points in the primary dataset and width allowing for up to N1 time segments.
%
%   EXAMPLES:
%       op = Aircraft_Collocation_Procedure(FN1,FN2,time_sep_min,distance_sep_km,N1) 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Script to match lidar data measured at one time and overlapped at aircraft
%position at another time.
function op = Aircraft_Collocation_Procedure(Aircraft1_Filename,Aircraft2_Filename,time_sep_min,distance_sep_km,N1)

% retireve data to be collocated with primary data position/times
V = importICARTT(Aircraft2_Filename);
j1 = find(contains(V.dataheaders,"Time_Start","IgnoreCase",true));
j2 = find(contains(V.dataheaders,"Latitude","IgnoreCase",true));
j3 = find(contains(V.dataheaders,"Longitude","IgnoreCase",true));
j4 = find(contains(V.dataheaders,"GPS_Altitude","IgnoreCase",true));

Aircraft2_time = V.data(:,j1);
Aircraft2_lat = V.data(:,j2);
Aircraft2_lon = V.data(:,j3);
Aircraft2_alt = V.data(:,j4);

%  retireve data that will serve as primary data
V = importICARTT(Aircraft1_Filename);
j1 = find(contains(V.dataheaders,"Time_Start","IgnoreCase",true));
j2 = find(contains(V.dataheaders,"Latitude","IgnoreCase",true));
j3 = find(contains(V.dataheaders,"Longitude","IgnoreCase",true));
j4 = find(contains(V.dataheaders,"GPS_Altitude","IgnoreCase",true));

Aircraft1_time = V.data(:,j1);
Aircraft1_lat = V.data(:,j2);
Aircraft1_lon = V.data(:,j3);
Aircraft1_alt = V.data(:,j4);

% Create matrix of zeros with length equal to the number of points in the primary dataset and width allowing for up to N1 time segments
output = NaN(length(Aircraft2_alt),N1); time_output=output;
h = waitbar(0,'Calculating...');

L1 = length(Aircraft2_alt);
% # start loop to collocate each point in the primary dataset with 
for i1=1:L1  
    waitbar(i1/L1); % progress tracker

    alt = Aircraft2_alt(i1);
    lat = Aircraft2_lat(i1);
    lon = Aircraft2_lon(i1);
    time = Aircraft2_time(i1);

    r = haversine([lat,lon],[Aircraft1_lat,Aircraft1_lon]);
    t = Aircraft1_time-time;
    a = alt - Aircraft1_alt;

    idx = r < distance_sep_km & abs(t) < time_sep_min*60;

    if sum(idx) > 0
        tmp = diff([0;idx]);
        num = sum(tmp==1); %unique segments
        if num>10
            num = 10;
        end
        start_idx = find(tmp==1); %start index of segment
        stop_idx = find(tmp == -1);%stop index of segment
        if length(stop_idx) ~= length(start_idx) %check for some end point issues
            if length(start_idx) == 1
                stop_idx = start_idx; %case where there was not an end transition for the seqment
            elseif isempty(start_idx)
                start_idx = 1;
            else
                stop_idx(length(start_idx)) = start_idx(length(start_idx)); %case where the last end segment did not have a transition
            end
        end
        for p1=1:num %number of segments
            %find min range of the segment
            min_r_seg=min(r(start_idx(p1):stop_idx(p1)));
            indx = find(r==min_r_seg);
            output(i1,p1) = min_r_seg;
            time_output(i1,p1) = Aircraft1_time(indx(1));
        end
    end
end
output(isnan(output)) = -9999; % After finishing the loop, fill remaining zeros with -9999
time_output(isnan(time_output)) = -9999;
close(h)
% return output as a structure containing the time stamp of the primary data, the distance between the data at each collocated segment, and the corresponding time of the collocated data at each collocated segment.
op.Time_Start = Aircraft2_time;
op.min_range = output;
op.time_index = time_output;
