########################################################################################################################
# ApplyCollocationMask.py                   		  by:  Joseph Schlosser
#                                                revised:  02 Jan 2023 
#                                    language (revision):  python3 (3.8.2-0ubuntu2)
# 
# DESCRIPTION: This code serves as an example of how to apply the Collocation Mask files. Each of the required inputs 
# forthis code are described as follows:
# 1) Falcon_Filename: String or character containing the name of the desired Falcon's ICARTT v2.0 (.ict) file.
# 2) collocation_mask_Filename: String or character containing the name of the desired data mask .ict file. 
#    -> Use primary platform: King Air.
# 3) RSP_Filename: String or character containing the name of the desired RSP netcdf file.
# 4) HSRL2_Filename: String or character containing the name of the desired HSRL-2 netcdf file.
# 5) additional_time_separation_constraint: double, float, or int of maximum temporal platform separation in minutes.
# 6) additional_spatial_separation_constraint: double, float, or int of required maximum spatial platform separation in 
#    km.
#
# The output of this code are the numpy arrays X_all and Y_all which are defined as follows:
# 1) X_all stores all collocated N_LAS data taken from Falcon data during the collocation procedure 
# 2) Y_all stores all collocated N_HSRL-RSP data taken from King Air data during the collocation procedure
#
# EXAMPLE:
#       Nlas, Nhsrlrsp = ApplyCollocationMask.CaseStudy("activate-mrg1_hu25_20200215_R0.ict",
#                                                       "ACTIVATE-CollocationMask-KingAir_Analysis_20200215_R0.ict",
#                                                       "ACTIVATE-RSP-AER_UC12_20200215_R02.h5",
#                                                       "ACTIVATE-HSRL2_UC12_20200215_R1",
#                                                       6,
#                                                       15)     
#       print(Nlas, Nhsrlrsp)   
#       size Nlas:[1653, 10]
#       size Nhsrlrsp:[1653, 10]   
#                      
# WARNINGS:
# 1) numpy, datetime, itertools, matplotlib, mpl_toolkits, and pylab must be installed to the python environment
# 2) importICARTT.py, StatsCode.py, collect_netcdf.py, merge_remote_sensor.py, amb_na.py, and LAS_sizebins.csv, files 
#    with the corresponding filenames must be present in a directory that is in your PATH
########################################################################################################################

import StatsCode
import importICARTT
import collect_netcdf
import merge_remote_sensor
import amb_na
import LAS_sizebins

import numpy as np
import pandas as pd
import datetime
import itertools
import matplotlib as mpl
import matplotlib.pyplot as plt 
from matplotlib.ticker import MaxNLocator
from matplotlib.dates import DayLocator, HourLocator, DateFormatter
from matplotlib.colors import LogNorm
from matplotlib import cm
from matplotlib.collections import PolyCollection
from matplotlib.colors import LogNorm
from matplotlib import cm
from matplotlib.collections import PolyCollection
from matplotlib.colors import ListedColormap, LinearSegmentedColormap
from matplotlib.offsetbox import AnchoredText
from mpl_toolkits.basemap import Basemap
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from pylab import rcParams

def CaseStudy(
    Falcon_Filename,
    collocation_mask_Filename,
    RSP_Filename,
    HSRL2_Filename,
    additional_time_separation_constraint,
    additional_spatial_separation_constraint
):
    
    fs = 36 # font size for the figures.
    N1 = 10 # number of possible segments to be consistent with the collocation procedure
    rsp_scn_dur = 60/72 # seconds per rsp scan 
    tres = 10 # native hsrl-2 extinction temporal resolution in seconds
    ares = 150 # native hsrl-2 extinction vertical resolution in meters
    altrg = np.arange(int(ares/2+ares*3),int(9000),int(ares)) # altitude grid for hsrl-2 data and omit bins 1, 2, and 3 

    # function to retrieve data using key from a dictionary
    def grab_keydata(data, key_name):
        for key in data.keys():
            if key.startswith(key_name):
                return data[key]   

    # function to retrieve time (SAM UTC), latitude (°N), longitude (°E), formatted time (YYYY:MM:DD hh:mm:ss UTC), 
    # aircraft altitude (m), date information, stdPT factor for LAS data (unitless), CDP droplet number concentration 
    # (cm-3), CDP liquid water content (g m-3), and intlet position flag (unitless) dictionary.         
    def grab_Data(dictionary):
        opdict = dict()
        opdict["time"] = np.array(grab_keydata(dictionary, 'Time_Start'))
        opdict["latitude"] = np.array(grab_keydata(dictionary, 'Latitude'))
        opdict["longitude"] = np.array(grab_keydata(dictionary, 'Longitude'))
        opdict["fmtdatetime"] = np.array(grab_keydata(dictionary, 'fmtdatetime'))
        opdict["GPS_Altitude"] = np.array(grab_keydata(dictionary, 'GPS_Altitude'))
        opdict["date"] = np.array(grab_keydata(dictionary, 'date'))
        opdict["stdPT"] = np.array(grab_keydata(dictionary, 'stdPT_ZIEMBA_'))
        opdict["Nd"] = np.array(grab_keydata(dictionary, 'N_CDP'))
        opdict["LWC"] = np.array(grab_keydata(dictionary, 'LWC_CDP'))
        opdict["InletFlag"] = np.array(grab_keydata(dictionary, 'InletFlag_LARGE'))      
        return opdict  

    # use the importICARTT procedure to retrieve the Falcon's data from .ict file and convert to a python dictionary    
    dictionary = importICARTT.imp(Falcon_Filename) 

    # use the grab_Data function and define separate variables for each desired key in the dictionary
    opdict = grab_Data(dictionary) 
    Aircraft0_time = opdict["time"]
    Aircraft0_latitude = opdict["latitude"]
    Aircraft0_longitude = opdict["longitude"]  
    Aircraft0_fmtdatetime = opdict["fmtdatetime"]
    Aircraft0_GPS_Altitude = opdict["GPS_Altitude"]
    Aircraft0_date = opdict["date"] 
    Aircraft0_stdPT = opdict["stdPT"]
    Aircraft0_Nd = opdict["Nd"]
    Aircraft0_LWC = opdict["LWC"]
    Aircraft0_InletFlag = opdict["InletFlag"]

    Aircraft0_Nd[Aircraft0_Nd.astype(str)=='nan']=0 # replace 'nan' with 0 in the Nd data
    Aircraft0_LWC[Aircraft0_LWC.astype(str)=='nan']=0 # replace 'nan' with 0 in the LWC data

    # retrieve LAS size resolved number concentration from the Falcon data dictionary
    sd1 = np.array([v for k, v in dictionary.items() if k.startswith('LAS_')])  

    # gather midpoint particle diamters corresponding to the LAS size distribution
    dpg = LAS_sizebins.Load('LAS_bin_sizes.csv') 

    # use the amb_na procedure to calculate N_LAS, S_LAS, and V_LAS from the LAS size distribution. midpoint diameters, 
    # and the stdPT factor.
    ambNVS = amb_na.cal(sd1,dpg,Aircraft0_stdPT)

    # flag all data that is not cloud-free (Nd>5 & LWC>0.001)
    IDX = np.where((Aircraft0_Nd>5)|(Aircraft0_LWC>0.001))[0] 
    IDx = np.zeros(len(ambNVS["Nlas"]))
    IDx[IDX] = 1

    # flag all data that is cloudy (Nd>50 & LWC>0.01)
    IDX2 = np.where((Aircraft0_Nd>50)|(Aircraft0_LWC>0.01))[0] 
    IDx2 = np.zeros(len(ambNVS["Nlas"]))
    IDx2[IDX2] = 1

    # flag all data where the inlet is set to the CVI (inlet flag > 0)
    IDX3 = np.where((Aircraft0_InletFlag>0))[0] 
    IDx3 = np.zeros(len(ambNVS["Nlas"]))
    IDx3[IDX3] = 1

    ambNVS["Nlas"][IDX] = 'nan' # remove all ambiguous and cloudy data from Nlas
    ambNVS["Nlas"][IDX3] = 'nan' # remove all CVI data from Nlas

    # convert RSP and HSRL-2 data from their netcdf files to dictionaries using the collect_netcdf procedures 
    RSP_Dictionary = collect_netcdf.grabRSP(RSP_Filename)
    HSRL2_Dictionary = collect_netcdf.grabHSRL2(HSRL2_Filename)

    # calculate N_HSRL-RSP using extinction cross section and extinction coefficient from RSP and HSRL-2, respectively
    nc = merge_remote_sensor.calVertNa(RSP_Dictionary,HSRL2_Dictionary,tres,altrg,ares)
    N2 = len(nc[0,:]) # assign variable to number of grid points of the Nhsrl-rsp data set

    # retrieve collocated data created from .ict file resulting from the collocation procedure using the King Air as the
    # primary platform
    Out_dictionary = importICARTT.imp(collocation_mask_Filename)

    # assign variable for the time and formatted time arrays from the data mask
    primary_time = grab_keydata(Out_dictionary, 'Time_Start') 
    fmt_primary_time = grab_keydata(Out_dictionary, 'fmtdatetime')

    # separate the time and spatial separation data into matricies containing one column for each time segment up to N1
    # segments 
    i1 = 0
    secondary_time = np.zeros((len(primary_time),N1))
    for key in Out_dictionary.keys():
    	if key.startswith('Falcon_Time_Start'):
    		secondary_time[:,i1] = Out_dictionary[key]
    		i1 = 1 + i1
    i1 = 0
    delta_distance = np.zeros((len(primary_time),N1))
    for key in Out_dictionary.keys():
    	if key.startswith('Aircraft_Separation'):
    		delta_distance[:,i1] = Out_dictionary[key]
    		i1 = 1 + i1

    # calculate and store the temporal separation for each time segment       
    delta_time = np.zeros((len(primary_time),N1))
    for i1 in range(N1):
    	delta_time[:,i1] = np.add(primary_time,-secondary_time[:,i1])#

    # apply additional temporal filter as desired and defined by user
    additionalFilter = np.where((np.abs(delta_time) > additional_time_separation_constraint*60))
    secondary_time[additionalFilter] = 'nan'
    delta_distance[additionalFilter] = 'nan'

    # apply additional spatial filter as desired and defined by user
    additionalFilter = np.where((delta_distance > additional_spatial_separation_constraint*1000))
    secondary_time[additionalFilter] = 'nan'
    delta_distance[additionalFilter] = 'nan'

    # assign variables for the formatted time (YYYY:MM:DD hh:mm:ss UTC) and time array (decimal HAM UTC) from RSP
    rsp_frmttimedata = np.array(RSP_Dictionary["rsp_frmttimedata"])
    rsp_timedata = np.array(RSP_Dictionary["rsp_time_array"])*3600 # convert decimal HAM to SAM

    # calculate and store RSP scan time using scan duration and number of scans 
    rsp_dur = rsp_scn_dur*int(RSP_Dictionary["rsp_navg_scn"])/2 

    L_timearray = len(rsp_timedata) # assign variable to number of points in the RSP data
    rsp_start_time = rsp_timedata - rsp_dur # calculate and assign variable for RSP scan start (SAM UCT)
    rsp_end_time = rsp_timedata + rsp_dur # calculate and assign variable for RSP scan end (SAM UCT)

    # for the altitude and Nlas from Falcon data, smooth to the RSP scan time resolution and sort into time segments 
    # sort Nhsrl-rsp into time segments
    Collocated_N = np.zeros((L_timearray,N1))
    Collocated_Alt = np.zeros((L_timearray,N1))
    Collocated_N2 = np.zeros((L_timearray,N1,N2))  
    for i2 in range(L_timearray):
        indx1 = np.where((primary_time<=rsp_end_time[i2])&(primary_time>rsp_start_time[i2]))
        idx1_sum = np.sum(indx1)
        if idx1_sum > 0:
            for i1 in range(N1):
                stime = np.squeeze(secondary_time[indx1,i1])
                stime = stime[stime.astype(str)!="nan"]
                indx = np.zeros(len(stime))
                for i3 in range(len(stime)):
                    ix = np.where(Aircraft0_time == stime[i3])[0]
                    indx[i3] = ix   
                if np.sum(indx)>0:
                    indx = indx.astype(int)
                    nlas = ambNVS["Nlas"][indx]
                    IDX_nlas = IDx[indx].astype('int')
                    IDX_nlas2 = IDx2[indx].astype('int')
                    IDX_nlas3 = IDx3[indx].astype('int')
                    if (np.sum(IDX_nlas)==0)&(np.sum(IDX_nlas3)==0):
                        afalc = Aircraft0_GPS_Altitude[indx]
                        Collocated_Alt[i2,i1] = np.nanmean(afalc[nlas.astype(str)!="nan"])
                        Collocated_N[i2,i1] = np.nanmean(nlas[nlas.astype(str)!="nan"])#
                        for i0 in range(len(altrg)):
                            if (Collocated_Alt[i2,i1] > altrg[i0] - ares/2) & 
                                (Collocated_Alt[i2,i1] <= altrg[i0] + ares/2):
                                Collocated_N2[i2,i1,i0] = Collocated_N[i2,i1]

    # reshape N_LAS and N_HSRL-RSP to be a single dimension each.                            
    X_all = None
    Y_all = None
    for i1 in range(N1):
        x = np.squeeze(Collocated_N2[:,i1,:])
        y = nc
        if X_all is not None:
            X_all = np.vstack((X_all,x.reshape(1,-1)))
            Y_all = np.vstack((Y_all,y.reshape(1,-1)))
        else:   
            X_all = np.array(x.reshape(1,-1))
            Y_all = np.array(y.reshape(1,-1))

    return X_all, Y_all
