%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% importICARTT.m                                      by:  Joseph Schlosser
%                                                revised:  2021 Oct 26.
%                                    language (revision):  MATLAB (2020a update 4)
% Description: M-file to open ICARTT file and format the data and headers into the output (op) structure.
% Each of the required inputs for this code are described as follows:
% 1) filename: String or character containing the name of the desired ICARTT (.ict) file.                  
%    
% The output of this code is op, which is a MATLAB structure that is described as follows: 
% 1) op.data: Numeric matrix containing all the data within the specified .ict file.
%    -> dimensions: [length of data set,number of variables in dataset]
% 2) op.dataheaders: String matrix containing all of names of the variables contained in the specified .ict file.
%    -> dimensions: [number of variables in dataset,1]
% 
%   EXAMPLE:
%       a = importICARTT('activate-mrg1_hu25_20200825_R0.ict');
%
%   WARNING: 
%   importICARTT.m and file with the corresponding filename must be present in a directory that is in your MATLABPATH in
%   order for this algorithm to execute properly.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function op = importICARTT(filename)
f = fopen(filename); % open file for reading        
g = textscan(f,'%s','delimiter','\n'); % scan text to cell array 
fclose(f); % close file 
fv = double(split(string(g{1,1}(12)),{', ',','})); % acquire fill values to be replaced with NAN 
varnames = string(g{1,1}([9 13:13+double(string((g{1,1}{10})))-1])); % Create variable name array 
vr_id = double(string(split(g{1,1}{1},{', ',','}))); % Index variables 
Vrs = g{1,1}(vr_id(1):end); % define variable data cell array 
AAa= string(zeros(length(varnames),3)); % definestring array to be filled with final variable names, units, and extra info
ACc =zeros(length(Vrs),length(varnames)); % define variable data array to be filled with final data 
% iteratively fill string array with final variable names, units, and extra info
for i1 = 1:length(varnames)
    aa = split(varnames(i1),", "); % split string by ", "
    for i2 = 1:length(aa)
        AAa(i1,i2) = aa(i2); % fill string array at each index
    end
end
% iteratively fill data array with data and replace fv values with NAN
for i1 = 1:length(Vrs)
    ACC = double(string(split(Vrs(i1),{', ',','}))); % split string by ", " and ','
    for i2 = 2:length(ACC)
        if ACC(i2)==fv(i2-1)
            ACC(i2) = nan; % replace value with nan if value == fv
        elseif ACC(i2)<=-999
            ACC(i2) = nan; % CHECK; values less than -999 should be replaced with NAN regardless of value of fv
        end
    end
    ACc(i1,:) = ACC; % fill data array at each index
end

time = ACc(:,1);
op.fmtdatetime = datetime(double(string(Bx(1:4))).*ones(length(time),1),...
    double(string(Bx(5:6))).*ones(length(time),1),double(string(Bx(7:8))).*ones(length(time),1),...
    zeros(length(time),1),zeros(length(time),1),time);
op.fmtdatetime = zeros(length(time),6);
for i1 = 1:length(time)
    Hrs = floor(time(i1)/(60*60));
    Mnts = floor((time(i1)/(60*60)-floor(time(i1)/(60*60)))*60);
    Secd = ((time(i1)/(60*60)-floor(time(i1)/(60*60)))*60-floor((time(i1)/(60*60)-floor(time(i1)/(60*60)))*60))*60;
    Yr = DATE(0);
    Mon = DATE(1);
    Day = DATE(2);
    % set day to next dat if hours are greater than 23
    if Hrs > 23
        Hrs = Hrs - 24;
        Day = Day + 1;
    end
    op.mattimedata(i1,:)  = [Yr,Mon,Day,Hrs,Mnts,Secd];
end
op.data = ACc; % save data array to structure for output
op.dataheaders = AAa(:,1)+"_"+AAa(:,2); % create and save variable name_unit array to structure for output
end
%%