########################################################################################################################
# merge_remote_sensor .py                             by:  Joseph Schlosser
#                                                revised:  02 Jan 2023 
#                                    language (revision):  python3 (3.8.2-0ubuntu2)
# 
# DESCRIPTION: This procedure merges the HSRL-2 and RSP data products to calculate N_HSRL-RSP following the methods 
# described in Schlosser et al. (2022). The require inputs are as follows:
# 1) Falcon_Filename: String or character containing the name of the desired Falcon's ICARTT v2.0 (.ict) file.
# 2) collocation_mask_Filename: String or character containing the name of the desired data mask .ict file. 
#    -> Primary platform: King Air.
# 3) RSP_Filename: String or character containing the name of the desired RSP netcdf file.
# 4) HSRL2_Filename: String or character containing the name of the desired HSRL-2 netcdf file.
# 5) additional_time_separation_constraint: required maximum temporal platform separation in minutes.
# 6) additional_spatial_separation_constraint: required maximum spatial platform separation in km.
#
# The output of this code is the numpy array nc, which stores all N_HSRL-RSP data.
#
#   EXAMPLE:
#          RSP_Dictionary = collect_netcdf.grabHSRL2("ACTIVATE-RSP-AER_UC12_20200215_R02.h5") 
#          HSRL2_Dictionary = collect_netcdf.grabHSRL2("ACTIVATE-HSRL2_UC12_20200215_R1") 
#          nc = ApplyCollocationMask.CaseStudy(RSP_Dictionary,
#                                              HSRL2_Dictionary,
#                                              time_res,
#                                              altrg,
#                                              alt_res)
#          print(f"size Nhsrlrsp:[{len(nc[0,:])}, {len(nc[:,0])}]") 
#          size Nhsrlrsp:[57, 29]            
# WARNINGS:
# 1) numpy must be installed to the python environment
########################################################################################################################

import numpy as np
def calVertNa(RSP_Dictionary,HSRL2_Dictionary,time_res,altrg,alt_res):
    kext_f = np.array(RSP_Dictionary["rsp_dictionary"]["aerosol_kext_f"])[:,8]
    rsp_aot_f = np.array(RSP_Dictionary["rsp_dictionary"]["aerosol_tau_f"])[:,8]
    rsp_aot_c = np.array(RSP_Dictionary["rsp_dictionary"]["aerosol_tau_c"])[:,8]
    rsp_aot_t = np.add(rsp_aot_f,rsp_aot_c)
    rsp_time_array = np.array(RSP_Dictionary["rsp_time_array"])

    hsrl2_aot_data = np.array(HSRL2_Dictionary["hsrl2_dictionary"]["DataProducts"]["532_AOT_hi"])
    hsrl2_extinction_data = np.array(HSRL2_Dictionary["hsrl2_dictionary"]["DataProducts"]["532_ext"]) # km-1
    hsrl2_time_array = np.array(HSRL2_Dictionary["hsrl2_time_array"])
    hsrl2_extinction_data[np.isnan(hsrl2_extinction_data)]=0
    hsrl2_altitude = np.array(HSRL2_Dictionary["hsrl2_dictionary"]["DataProducts"]["Altitude"]) 
    L1 = len(rsp_time_array)
    L2 = len(altrg)#
    nc = np.zeros((L1,L2))
    altidx = dict()
    for i2 in range(L2):
        altidx[f'{i2}'] = np.where((hsrl2_altitude<=altrg[i2]+alt_res/2)&(hsrl2_altitude>altrg[i2]-alt_res/2))[1]
    for i1 in range(L1):
        j1 = np.where(np.sum(hsrl2_extinction_data,1)>0)[0]
        tdif = abs(hsrl2_time_array[j1]-rsp_time_array[i1])
        j2 = np.where((tdif<=np.nanmin(tdif)+time_res/3600)&(tdif>=np.nanmin(tdif)-time_res/3600))[0]
        if j2.all() != 0:
            kext = kext_f[i1]
            aotf = rsp_aot_f[i1]
            aott = rsp_aot_t[i1]
            hsrlaot = hsrl2_aot_data[j1[j2]]
            hsrlaot[hsrlaot<0]=np.nan
            hsrlext = hsrl2_extinction_data[j1[j2],:]
            hsrlext[hsrlext<0]=np.nan
            for i2 in range(len(hsrlaot)):
                flg = abs(aott-hsrlaot[i2])
                flg2 =  abs(aotf-hsrlaot[i2])
                cutoff = hsrlaot[i2]*0.5
                if cutoff < 0.05:
                    if (flg > cutoff) or (flg2 > 0.1):
                        hsrlaot[i2] = np.nan
                        hsrlext[i2,:] = np.nan
                else:
                    if (flg > 0.05) or (flg2 > 0.1):
                        hsrlaot[i2] = np.nan
                        hsrlext[i2,:] = np.nan         
            hsrlaot = np.nanmean(hsrlaot)
            hsrlext = np.nanmean(hsrlext,axis=0)
            smoothed_hsrlext = np.zeros(L2)
            for i2 in range(L2):
                smoothed_hsrlext[i2] = np.nanmean(hsrlext[altidx[f'{i2}']])
                nc[i1,i2] = smoothed_hsrlext[i2]*pow(10,3)/kext  
    return nc